xquery version "3.1";

(:~
 : This module provides the API endpoint for sending email.
 : Parameters are expected from html form send via POST request.
 : This file replaces the php backend of the highland template (https://gitlab.gwdg.de/SADE/template/-/tree/master)
 :
 : Output format is orientated at the expected json for the highland template, 
 : with a slight modification: error for email/name/message are stored in one object literal as key/valuepairs; 
 : the original highland json uses nested literals for error (key) and email/name/message (key in nested literal))
 :
 : @author Florian Barth
 : @version 1.0
 :)

module namespace semapi="http://architrave.eu/ns/semapi";

import module namespace rest="http://exquery.org/ns/restxq";
import module namespace console="http://exist-db.org/xquery/console";

declare namespace mail="http://exist-db.org/xquery/mail";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";

declare 
  %rest:path("/api/form") 
  %rest:POST
  %rest:form-param("email", "{$email}", "unknown")
  %rest:form-param("name", "{$name}", "unknown")
  %rest:form-param("message", "{$message}", "unknown")
  %rest:form-param("lang", "{$lang}", "unknown")
  %output:method("json")
  function semapi:sendmail(
    $message,
    $email,
    $name,
    $lang
    ) as map()* {
  
    let $error_name := 
      if ($lang = "de") then "Fehler: Bitte geben Sie Ihren Namen an."
      else if ($lang = "fr") then "Erreur : Veuillez indiquer votre nom."
      else ()

    let $error_email := 
      if ($lang = "de") then "Fehler: Bitte geben Sie eine gültige Email-Adresse an."
      else if ($lang = "fr") then "Erreur : Veuillez saisir une adresse électronique valide."
      else ()

    let $error_message := 
      if ($lang = "de") then "Fehler: Bitte geben Sie Ihre Nachricht ein."
      else if ($lang = "fr") then "Erreur : Veuillez entrer votre message."
      else ()

    let $error := 
      if ($lang = "de") then "Fehler: Die Nachricht konnte nicht gesendet werden."
      else if ($lang = "fr") then "Erreur : Le message n'a pas pu être envoyé."
      else ()

    let $mail := <mail>
                    <from>contact@architrave.eu</from>
                    <reply-to/>
                    <to>florian.barth@uni-goettingen.de</to>
                    <bcc/>
                    <subject>ARCHITRAVE Contact Request</subject>
                    <message>
                        <xhtml>
                          <html>
                              <head>
                                <title>ARCHITRAVE Contact Request</title>
                              </head>
                              <body>
                                  <h3>New message from  { $name } ({ $email })</h3>
                                  <p>{$message}</p>
                              </body>
                          </html>
                        </xhtml>
                    </message>
                    </mail>

    let $json_map := 
      if (string-length($name) = 0) then
        map { "error_name": $error_name }
      else if (string-length($email) = 0) then
        map { "error_email": $error_email }
      else if (string-length($message) = 0) then
        map { "error_message": $error_message}
      else if ( mail:send-email($mail, (), "UTF-8") ) then
        map { "success": "Nachricht gesendet.",
              "name": $name,
              "email": $email,
              "message": $message
        }
      else
        map { "error": $error}

    return
        $json_map

};

